% Matrice de Masse locale et globale sur Gamma
% en utilisant siMeshElt.Masse
close all
clear all

meshfile=fc_oogmsh.gmsh.buildmesh2d('./geofile/square6dom4holes.geo',5);
Th=fc_simesh.siMesh(meshfile); % maillage global


u=@(x,y) cos(x+y-pi/3);
v=@(x,y) sin(x-y+1);

% integrale en x de u.v sur bord [a,b] avec Sage:
% ---------- sage ----------
% var('x,y,a,b,c,d')
% u(x,y)=cos(x+y-pi/3);v(x,y)=sin(x-y+1);
% integrate(u(x,y)*v(x,y),(x,a,b))
%---------------------------
Ixfun=@(a,b,y) (a-b)*sin(2*y-1-pi/3)/2 + cos(2*a+1-pi/3)/4 - cos(2*b+1-pi/3)/4;
% integrale en y de u.v sur bord [c,d] avec Sage:
% ---------- sage ----------
% var('x,y,a,b,c,d')
% u(x,y)=cos(x+y-pi/3);v(x,y)=sin(x-y+1);
% integrate(u(x,y)*v(x,y),(y,c,d))
%---------------------------
Iyfun=@(c,d,x) (d-c)*sin(2*x+1-pi/3)/2 - cos(2*c-1-pi/3)/4 + cos(2*d-1-pi/3)/4;


% integrale de u.v sur bord de [a,b]x[c,d] avec sage
Iexfun=@(a,b,c,d) Ixfun(a,b,c)+Ixfun(a,b,d)+Iyfun(c,d,a)+Iyfun(c,d,b);
IexfunEh=@(Eh) Iexfun(Eh.bbox(1),Eh.bbox(2),Eh.bbox(3),Eh.bbox(4));

d=1;
idx=Th.find(d);

%%% (1)
%%%   a. calcul integrale de u.v sur Gamma(20)
lab=20;
ML=siMesh.Masse(Th,'d',d,'labels',lab,'local',true);
UL=siMesh.eval(Th,u,'d',d,'labels',lab,'local',true);
VL=siMesh.eval(Th,v,'d',d,'labels',lab,'local',true);
IL=dot(ML*UL,VL);

i=Th.find(d,lab);
Iex=IexfunEh(Th.sTh{i});
fprintf('(1) a.  Iex=%.5e, |IL-Iex|=%.6e \n',Iex,abs(Iex-IL))

MG=siMesh.Masse(Th,'d',d,'labels',lab,'local',false); % Matrice globale
UG=siMesh.eval(Th,u,'d',d,'labels',lab,'local',false);
VG=siMesh.eval(Th,v,'d',d,'labels',lab,'local',false);
IG=dot(MG*UG,VG);

fprintf('        |IL-IG|=%.6e\n',abs(IL-IL))

%%%   b. calcul integrale de u.v sur Gamma(10+20)
labels=[10,20];
Iex=0; 
for lab=labels % calcul exact
  i=Th.find(d,lab);
  Iex=Iex+IexfunEh(Th.sTh{i});
end

ML=siMesh.Masse(Th,'d',d,'labels',labels,'local',true);
UL=siMesh.eval(Th,u,'d',d,'labels',labels,'local',true);
VL=siMesh.eval(Th,v,'d',d,'labels',labels,'local',true);
IL=0; % Calculs locaux
for i=1:length(labels)
  IL=IL+dot(ML{i}*UL{i},VL{i});
end
fprintf('(1) b.  Iex=%.5e, |IL-Iex|=%.6e \n',Iex,abs(Iex-IL))

% Calculs globaux (methode 1)
IG=0; 
for lab=labels
  i=Th.find(2,lab);
  MG=siMesh.Masse(Th,'d',d,'labels',lab,'local',false);
  UG=siMesh.eval(Th,u,'d',d,'labels',lab,'local',false);
  VG=siMesh.eval(Th,v,'d',d,'labels',lab,'local',false);
  IG=IG+dot(MG*UG,VG);
end
fprintf('        |IL-IG|=%.6e  [IG, methode 1]\n',abs(IG-IL))

% Calculs globaux (methode 2) hyp. u et v continues sur Omega
MG=siMesh.Masse(Th,'d',d,'labels',labels,'local',false);
UG=Th.eval(u);
VG=Th.eval(v);
IG=dot(MG*UG,VG);
fprintf('        |IL-IG|=%.6e  [IG, methode 2]\n',abs(IG-IL))

% Calculs globaux (methode 3) hyp. u et v continues sur Omega
MG=siMesh.Masse(Th,'d',d,'labels',labels,'local',false);
UG=siMesh.eval(Th,u,'d',d,'labels',labels,'local',false);
VG=siMesh.eval(Th,v,'d',d,'labels',labels,'local',false);
IG=dot(MG*UG,VG);
fprintf('        |IL-IG|=%.6e  [IG, methode 3]\n',abs(IG-IL))


%%%   c. calcul integrale de u.v sur Gamma

% calcul exact
[idx,labels]=Th.find(d);
Iex=0; 
for i=idx
  Iex=Iex+IexfunEh(Th.sTh{i});
end

% Calculs locaux
ML=siMesh.Masse(Th,'d',d,'labels',labels,'local',true);
UL=siMesh.eval(Th,u,'d',d,'labels',labels,'local',true);
VL=siMesh.eval(Th,v,'d',d,'labels',labels,'local',true);
IL=0;
for i=1:length(labels)
  IL=IL+dot(ML{i}*UL{i},VL{i});
end
fprintf('(1) c.  Iex=%.5e, |IL-Iex|=%.6e \n',Iex,abs(Iex-IL))

% Calculs globaux (methode 1)
IG=0; 
for lab=labels
  MG=siMesh.Masse(Th,'d',d,'labels',lab,'local',false);
  UG=siMesh.eval(Th,u,'d',d,'labels',lab,'local',false);
  VG=siMesh.eval(Th,v,'d',d,'labels',lab,'local',false);
  IG=IG+dot(MG*UG,VG);
end
fprintf('        |IL-IG|=%.6e  [IG, methode 1]\n',abs(IG-IL))

% Calculs globaux (methode 2) hyp. u et v continues sur Gamma
MG=siMesh.Masse(Th,'d',d,'labels',labels,'local',false);
UG=Th.eval(u);
VG=Th.eval(v);
IG=dot(MG*UG,VG);
fprintf('        |IL-IG|=%.6e  [IG, methode 2]\n',abs(IG-IL))

% Calculs globaux (methode 3) hyp. u et v continues sur Gamma
MG=siMesh.Masse(Th,'d',d,'labels',labels,'local',false);
UG=siMesh.eval(Th,u,'d',d,'labels',labels,'local',false);
VG=siMesh.eval(Th,v,'d',d,'labels',labels,'local',false);
IG=dot(MG*UG,VG);
fprintf('        |IL-IG|=%.6e  [IG, methode 3]\n',abs(IG-IL))
